<?php
// @codeCoverageIgnoreStart
// {{{ICINGA_LICENSE_HEADER}}}
/**
 * This file is part of Icinga Web 2.
 *
 * Icinga Web 2 - Head for multiple monitoring backends.
 * Copyright (C) 2013 Icinga Development Team
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * @copyright  2013 Icinga Development Team <info@icinga.org>
 * @license    http://www.gnu.org/licenses/gpl-2.0.txt GPL, version 2
 * @author     Icinga Development Team <info@icinga.org>
 *
 */
// {{{ICINGA_LICENSE_HEADER}}}

use Icinga\Web\Controller\BasePreferenceController;
use Icinga\Web\Widget\Tab;
use Icinga\Application\Config as IcingaConfig;
use Icinga\Web\Url;
use Icinga\Form\Preference\GeneralForm;

/**
 * Application wide preference controller for user preferences
 */
class PreferenceController extends BasePreferenceController
{
    /**
     * Create tabs for this preference controller
     *
     * @return  array
     *
     * @see     BasePreferenceController::createProvidedTabs()
     */
    public static function createProvidedTabs()
    {
        return array(
            'general' => new Tab(
                array(
                    'title'     => 'General settings',
                    'url'       => Url::fromPath('/preference')
                )
            )
        );
    }

    /**
     * General settings for date and time
     */
    public function indexAction()
    {
        $form = new GeneralForm();
        $this->getTabs()->activate('general');
        $form->setConfiguration(IcingaConfig::app())
            ->setRequest($this->getRequest());
        if ($form->isSubmittedAndValid()) {
            try {
                $this->savePreferences($form->getPreferences());
                $this->view->successMessage = 'Preferences Updated Successfully';
                // Recreate form to show new values
                // TODO(el): It must sufficient to call $form->populate(...)
                $form = new GeneralForm();
                $form->setConfiguration(IcingaConfig::app());
                $form->setRequest($this->getRequest());
            } catch (Exception $e) {
                $this->view->exceptionMessage = $e->getMessage();
            }
        }
        $this->view->form = $form;
    }
}
// @codeCoverageIgnoreEnd
